/*******************************************************
*
* This is a modified version of the PHP wrapper from
* Alexander Amelkin.
* 
* I modified it to run under conditions where php is
* running outside cgi-bin.
* 
* Now the wrapper fixes the environmentvariables
* SCRIPT_NAME and SCRIPT_FILENAME.
*
* Wed Aug 21 16:46:46 CEST 2002
* Simon "janus" Dassow, janus@linux-de.org
*
*
* (c) 2002, Alexander Amelkin, spirit@reactor.ru
*
* -----------------------------------------------------
* To enable suexec php scripts, first of all you must
* disable php processing by mod_php. To do so, comment
* out all php-related strings in your httpd.conf
*
* Add the following strings to httpd.conf:
*  AddType application/x-httpd-php .php .php4 .php3 .phtml
*  Action application/x-httpd-php /cgi-bin/php_cgi        
*
* Do the following:
* gcc php_cgi.c -o php_cgi
* chmod 755 php_cgi
*
* Copy (do not use ln) php_cgi to the ScriptAlias /cgi-bin/ directory
* of each VirtualHost, where php must be enabled. chown php_cgi
* to whatever is set by User and Group directives for each
* VirtualHost
*
* You're done. No modification to the existing php scripts
* is required.
* 
*******************************************************/


#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// PHP binary file
#define PHPPATH "/usr/local/bin/php"

int main() {
	char *buf;
	char *script_name;
	char *script_filename;
	char *document_root;

	// Check if a script name was supplied
	if(getenv("PATH_TRANSLATED")!=NULL) {
		// Allocate some memory for a temporary buffer
		buf=(char*)malloc(strlen(getenv("PATH_TRANSLATED")));
		if(buf==NULL) {
			printf("Content-Type: text/plain\n\n");
			perror("Memory allocation error");
			return 1;
		}

		// Get the script's working directory
		strcpy(buf,getenv("PATH_TRANSLATED"));
		rindex(buf,'/')[0]=0;

		// chdir there
		if(chdir(buf)==-1) {
			printf("Content-Type: text/plain\n\n");
			perror("Can't chdir");
			return 1;
		}
		// We don't need the buffer anymore
		free(buf);

		// Reset broken SCRIPT_* vars
		// Allocate memory
		script_name=(char*)malloc(strlen(getenv("REQUEST_URI")));
		document_root=(char*)malloc(strlen(getenv("DOCUMENT_ROOT")));
		if((script_name==NULL)||(document_root==NULL)) {
			printf("Content-Type: text/plain\n\n");
			perror("Memory allocation error");
			return 1;
		}
		strcpy(script_name,getenv("REQUEST_URI"));
		strcpy(document_root,getenv("DOCUMENT_ROOT"));
		setenv("REQUEST_URI",script_name,1);
		if(strchr(script_name,'?')) rindex(script_name,'?')[0]=0;
		setenv("SCRIPT_NAME",script_name,1);
		script_filename=(char*)malloc((strlen(document_root)+strlen(script_name)+1));
		if(script_filename==NULL) {
			printf("Content-Type: text/plain\n\n");
			perror("Memory allocation error");
			return 1;
		}
		sprintf(script_filename,"%s%s",document_root,script_name);
		setenv("SCRIPT_FILENAME",script_filename,1);
		free(script_name);
		free(script_filename);
		free(document_root);

		// Invoke php to run the script, all CGI parameters are left
		// untouched in the system environment
		execl(PHPPATH,PHPPATH,getenv("PATH_TRANSLATED"),NULL);
	}
	printf("Content-Type: text/plain\n\n");
	perror("Wrong script requested");
	return 1;
}
